package sun.net.www;

import java.util.NoSuchElementException;
import java.io.PrintStream;
import java.util.Collections;
import java.util.Collection;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Iterator;
import java.util.StringJoiner;
import java.io.IOException;
import java.io.InputStream;

public class MessageHeader
{
    private String[] keys;
    private String[] values;
    private int nkeys;
    
    public MessageHeader() {
        this.grow();
    }
    
    public MessageHeader(InputStream inputStream) throws IOException {
        this.parseHeader(inputStream);
    }
    
    public synchronized String getHeaderNamesInList() {
        StringJoiner stringJoiner = new StringJoiner(",");
        for (int i = 0; i < this.nkeys; ++i) {
            stringJoiner.add(this.keys[i]);
        }
        return stringJoiner.toString();
    }
    
    public synchronized void reset() {
        this.keys = null;
        this.values = null;
        this.nkeys = 0;
        this.grow();
    }
    
    public synchronized String findValue(String s) {
        if (s == null) {
            int nkeys2 = this.nkeys;
            while (--nkeys2 >= 0) {
                if (this.keys[nkeys2] != null) {
                    continue;
                }
                return this.values[nkeys2];
            }
        }
        else {
            int nkeys = this.nkeys;
            while (--nkeys >= 0) {
                if (!s.equalsIgnoreCase(this.keys[nkeys])) {
                    continue;
                }
                return this.values[nkeys];
            }
        }
        return null;
    }
    
    public synchronized int getKey(String s) {
        int nkeys = this.nkeys;
        while (--nkeys >= 0) {
            if (this.keys[nkeys] != s && (s == null || !s.equalsIgnoreCase(this.keys[nkeys]))) {
                continue;
            }
            return nkeys;
        }
        return -1;
    }
    
    public synchronized String getKey(int n) {
        if (n < 0 || n >= this.nkeys) {
            return null;
        }
        return this.keys[n];
    }
    
    public synchronized String getValue(int n) {
        if (n < 0 || n >= this.nkeys) {
            return null;
        }
        return this.values[n];
    }
    
    public synchronized String findNextValue(String s, String s2) {
        int n = 0;
        if (s == null) {
            int nkeys2 = this.nkeys;
            while (--nkeys2 >= 0) {
                if (this.keys[nkeys2] != null) {
                    continue;
                }
                if (n != 0) {
                    return this.values[nkeys2];
                }
                if (this.values[nkeys2] != s2) {
                    continue;
                }
                n = 1;
            }
        }
        else {
            int nkeys = this.nkeys;
            while (--nkeys >= 0) {
                if (!s.equalsIgnoreCase(this.keys[nkeys])) {
                    continue;
                }
                if (n != 0) {
                    return this.values[nkeys];
                }
                if (this.values[nkeys] != s2) {
                    continue;
                }
                n = 1;
            }
        }
        return null;
    }
    
    public boolean filterNTLMResponses(String s) {
        boolean n = false;
        int i = 0;
        while (i < this.nkeys) {
            if (s.equalsIgnoreCase(this.keys[i]) && this.values[i] != null && this.values[i].length() > 5 && this.values[i].substring(0, 5).equalsIgnoreCase("NTLM ")) {
                n = true;
                break;
            }
            ++i;
        }
        if (!(n == 0)) {
            int nkeys = 0;
            for (int j = 0; j < this.nkeys; ++j) {
                if (s.equalsIgnoreCase(this.keys[j])) {
                    if (!"Negotiate".equalsIgnoreCase(this.values[j])) {
                        if ("Kerberos".equalsIgnoreCase(this.values[j])) {
                            continue;
                        }
                    }
                    else {
                        continue;
                    }
                }
                if (j != nkeys) {
                    this.keys[nkeys] = this.keys[j];
                    this.values[nkeys] = this.values[j];
                }
                ++nkeys;
            }
            if (nkeys != this.nkeys) {
                this.nkeys = nkeys;
                return true;
            }
        }
        return false;
    }
    
    public Iterator<String> multiValueIterator(String s) {
        return new MessageHeader.HeaderIterator(s, this);
    }
    
    public synchronized Map<String, List<String>> getHeaders() {
        return this.getHeaders((String[])null);
    }
    
    public synchronized Map<String, List<String>> getHeaders(String[] array) {
        return this.filterAndAddHeaders(array, null);
    }
    
    public synchronized Map<String, List<String>> filterAndAddHeaders(String[] array, Map<String, List<String>> map) {
        int n = 0;
        HashMap<String, Object> m = new HashMap();
        int nkeys = this.nkeys;
        while (--nkeys >= 0) {
            if (array != null) {
                int i = 0;
                while (i < array.length) {
                    if (array[i] != null && array[i].equalsIgnoreCase(this.keys[nkeys])) {
                        n = 1;
                        break;
                    }
                    ++i;
                }
            }
            if (n == 0) {
                List<Object> list = (Object)m.get(this.keys[nkeys]);
                if (list == null) {
                    list = new ArrayList();
                    m.put(this.keys[nkeys], list);
                }
                list.add(this.values[nkeys]);
            }
            else {
                n = 0;
            }
        }
        if (map != null) {
            for (Map.Entry entry : map.entrySet()) {
                List<Object> list2 = (Object)m.get(entry.getKey());
                if (list2 == null) {
                    list2 = new ArrayList();
                    m.put(entry.getKey(), list2);
                }
                list2.addAll((Collection)entry.getValue());
            }
        }
        for (String s : m.keySet()) {
            m.put(s, Collections.unmodifiableList((List)m.get(s)));
        }
        return Collections.unmodifiableMap(m);
    }
    
    private boolean isRequestline(String s) {
        String trim = s.trim();
        int lastIndex = trim.lastIndexOf(32);
        if (lastIndex <= 0) {
            return false;
        }
        int length = trim.length();
        if (length - lastIndex < 9) {
            return false;
        }
        int char1 = trim.charAt(length - 3);
        int char2 = trim.charAt(length - 2);
        int char3 = trim.charAt(length - 1);
        return char1 >= '1' && char1 <= '9' && char2 == '.' && char3 >= '0' && char3 <= '9' && trim.substring(lastIndex + 1, length - 3).equalsIgnoreCase("HTTP/");
    }
    
    public synchronized void print(PrintStream printStream) {
        for (int i = 0; i < this.nkeys; ++i) {
            if (this.keys[i] != null) {
                StringBuilder o = new StringBuilder(this.keys[i]);
                if (this.values[i] != null) {
                    o.append(": " + this.values[i]);
                }
                else if (i != 0 || !this.isRequestline(this.keys[i])) {
                    o.append(":");
                }
                printStream.print(o.append("\r\n"));
            }
        }
        printStream.print("\r\n");
        printStream.flush();
    }
    
    public synchronized void add(String s, String s2) {
        this.grow();
        this.keys[this.nkeys] = s;
        this.values[this.nkeys] = s2;
        ++this.nkeys;
    }
    
    public synchronized void prepend(String s, String s2) {
        this.grow();
        for (int i = this.nkeys; i > 0; --i) {
            this.keys[i] = this.keys[i - 1];
            this.values[i] = this.values[i - 1];
        }
        this.keys[0] = s;
        this.values[0] = s2;
        ++this.nkeys;
    }
    
    public synchronized void set(int n, String s, String s2) {
        this.grow();
        if (n < 0) {
            return;
        }
        if (n >= this.nkeys) {
            this.add(s, s2);
        }
        else {
            this.keys[n] = s;
            this.values[n] = s2;
        }
        return;
    }
    
    private void grow() {
        if (this.keys == null || this.nkeys >= this.keys.length) {
            String[] keys = new String[this.nkeys + 4];
            String[] values = new String[this.nkeys + 4];
            if (this.keys != null) {
                System.arraycopy(this.keys, 0, keys, 0, this.nkeys);
            }
            if (this.values != null) {
                System.arraycopy(this.values, 0, values, 0, this.nkeys);
            }
            this.keys = keys;
            this.values = values;
        }
    }
    
    public synchronized void remove(String s) {
        if (s == null) {
            for (int j = 0; j < this.nkeys; ++j) {
                while (this.keys[j] == null && j < this.nkeys) {
                    for (; j < this.nkeys - 1; ++j) {
                        this.keys[j] = this.keys[j + 1];
                        this.values[j] = this.values[j + 1];
                    }
                    --this.nkeys;
                }
            }
        }
        else {
            for (int i = 0; i < this.nkeys; ++i) {
                while (s.equalsIgnoreCase(this.keys[i]) && i < this.nkeys) {
                    for (; i < this.nkeys - 1; ++i) {
                        this.keys[i] = this.keys[i + 1];
                        this.values[i] = this.values[i + 1];
                    }
                    --this.nkeys;
                }
            }
        }
    }
    
    public synchronized void set(String s, String s2) {
        int nkeys = this.nkeys;
        while (--nkeys >= 0) {
            if (!s.equalsIgnoreCase(this.keys[nkeys])) {
                continue;
            }
            this.values[nkeys] = s2;
            return;
        }
        this.add(s, s2);
    }
    
    public synchronized void setIfNotSet(String s, String s2) {
        if (this.findValue(s) == null) {
            this.add(s, s2);
        }
    }
    
    public static String canonicalID(String s) {
        if (s == null) {
            return "";
        }
        int n = 0;
        int length = s.length();
        boolean n2 = false;
        int char1;
        while (n < length && ((char1 = s.charAt(n)) == '<' || char1 <= ' ')) {
            ++n;
            n2 = true;
        }
        int char2;
        while (n < length && ((char2 = s.charAt(length - 1)) == '>' || char2 <= ' ')) {
            --length;
            n2 = true;
        }
        return n2 ? s.substring(n, length) : s;
    }
    
    public void parseHeader(InputStream inputStream) throws IOException {
        synchronized (this) {
            this.nkeys = 0;
        }
        this.mergeHeader(inputStream);
    }
    
    public void mergeHeader(InputStream inputStream) throws IOException {
        if (inputStream == null) {
            return;
        }
        char[] array = new char[10];
        int n = inputStream.read();
    Label_0015:
        while (n != 10 && n != 13 && n >= 0) {
            int n2 = 0;
            int n3 = -1;
            int n4 = (n > 32) ? 1 : 0;
            array[n2++] = (char)n;
            Label_0252: {
                int read;
                while ((read = inputStream.read()) >= 0) {
                    switch (read) {
                        case 58: {
                            if (n4 != 0 && n2 > 0) {
                                n3 = n2;
                            }
                            n4 = 0;
                            break;
                        }
                        case 9: {
                            read = 32;
                        }
                        case 32: {
                            n4 = 0;
                            break;
                        }
                        case 10:
                        case 13: {
                            n = inputStream.read();
                            if (read == 13 && n == 10) {
                                n = inputStream.read();
                                if (n == 13) {
                                    n = inputStream.read();
                                }
                            }
                            if (n != 10 && n != 13 && n <= 32) {
                                read = 32;
                                break;
                            }
                            while (n2 > 0 && array[n2 - 1] <= ' ') {
                                --n2;
                            }
                            String copyValue;
                            if (n3 <= 0) {
                                copyValue = null;
                                n3 = 0;
                            }
                            else {
                                copyValue = String.copyValueOf(array, 0, n3);
                                if (n3 < n2 && array[n3] == ':') {
                                    ++n3;
                                }
                                while (n3 < n2 && array[n3] <= ' ') {
                                    ++n3;
                                }
                            }
                            Object copyValue2;
                            if (n3 >= n2) {
                                copyValue2 = new String();
                            }
                            else {
                                copyValue2 = String.copyValueOf(array, n3, n2 - n3);
                            }
                            this.add(copyValue, copyValue2);
                            continue Label_0015;
                        }
                    }
                    if (n2 >= array.length) {
                        char[] array2 = new char[array.length * 2];
                        System.arraycopy(array, 0, array2, 0, n2);
                        array = array2;
                    }
                    array[n2++] = (char)read;
                }
                n = -1;
                continue;
            }
        }
        return;
    }
    
    @Override
    public synchronized String toString() {
        String str = super.toString() + this.nkeys + " pairs: ";
        for (int n = 0; n < this.keys.length && n < this.nkeys; ++n) {
            str = str + "{" + this.keys[n] + ": " + this.values[n] + "}";
        }
        return str;
    }
    
    class HeaderIterator implements Iterator<String>
    {
        int index;
        int next;
        String key;
        boolean haveNext;
        Object lock;
        
        public HeaderIterator(String key, Object lock) {
            this.index = 0;
            this.next = -1;
            this.haveNext = false;
            this.key = key;
            this.lock = lock;
        }
        
        @Override
        public boolean hasNext() {
            synchronized (this.lock) {
                if (this.haveNext) {
                    return true;
                }
                while (this.index < MessageHeader.this.nkeys) {
                    if (this.key.equalsIgnoreCase(MessageHeader.this.keys[this.index])) {
                        this.haveNext = true;
                        this.next = this.index++;
                        return true;
                    }
                    ++this.index;
                }
                return false;
            }
        }
        
        @Override
        public String next() {
            synchronized (this.lock) {
                if (this.haveNext) {
                    this.haveNext = false;
                    return MessageHeader.this.values[this.next];
                }
                if (this.hasNext()) {
                    return this.next();
                }
                throw new NoSuchElementException("No more elements");
            }
        }
        
        @Override
        public void remove() {
            throw new UnsupportedOperationException("remove not allowed");
        }
    }
}
